# Generated by Django 2.2.6 on 2019-10-14 17:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('file', '0002_file_related_names'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filecontent',
            name='relative_path',
            field=models.TextField(),
        ),
    ]
