# Generated by Django 2.2.17 on 2020-12-05 18:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('file', '0007_filefilesystemexporter'),
    ]

    operations = [
        migrations.AddField(
            model_name='filepublication',
            name='manifest',
            field=models.TextField(default='PULP_MANIFEST'),
            preserve_default=False,
        ),
    ]
