# Generated by Django 2.2.6 on 2020-03-13 16:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('file', '0005_filerepository'),
    ]

    operations = [
        migrations.DeleteModel(
            name='FileFileSystemExporter',
        ),
    ]
